#!/usr/bin/env python
import logging
import os
from os.path import abspath, dirname

from edc_test_utils import DefaultTestSettings, func_main

app_name = "edc_dashboard"
base_dir = dirname(abspath(__file__))

project_settings = DefaultTestSettings(
    calling_file=__file__,
    template_dirs=[os.path.join(base_dir, app_name, "tests", "templates")],
    APP_NAME=app_name,
    BASE_DIR=base_dir,
    ETC_DIR=os.path.join(base_dir, app_name, "tests", "etc"),
    EDC_NAVBAR_DEFAULT=app_name,
    EDC_AUTH_SKIP_SITE_AUTHS=True,
    EDC_AUTH_SKIP_AUTH_UPDATER=True,
    INSTALLED_APPS=[
        "django.contrib.admin",
        "django.contrib.auth",
        "django.contrib.contenttypes",
        "django.contrib.sessions",
        "django.contrib.messages",
        "django.contrib.staticfiles",
        "django.contrib.sites",
        "django_crypto_fields.apps.AppConfig",
        "django_revision.apps.AppConfig",
        "multisite",
        "edc_appointment.apps.AppConfig",
        "edc_action_item.apps.AppConfig",
        "edc_adverse_event.apps.AppConfig",
        "edc_crf.apps.AppConfig",
        "edc_notification.apps.AppConfig",
        "edc_consent.apps.AppConfig",
        "edc_data_manager.apps.AppConfig",
        "edc_device.apps.AppConfig",
        "edc_export.apps.AppConfig",
        "edc_identifier.apps.AppConfig",
        "edc_lab.apps.AppConfig",
        "edc_list_data.apps.AppConfig",
        "edc_locator.apps.AppConfig",
        "edc_metadata.apps.AppConfig",
        "edc_model_wrapper.apps.AppConfig",
        "edc_navbar.apps.AppConfig",
        "edc_offstudy.apps.AppConfig",
        "edc_pharmacy.apps.AppConfig",
        "edc_protocol.apps.AppConfig",
        "edc_randomization.apps.AppConfig",
        "edc_registration.apps.AppConfig",
        "edc_sites.apps.AppConfig",
        "edc_timepoint.apps.AppConfig",
        "edc_visit_schedule.apps.AppConfig",
        "edc_visit_tracking.apps.AppConfig",
        "edc_dashboard.apps.AppConfig",
        "adverse_event_app.apps.AppConfig",
        "edc_auth.apps.AppConfig",
    ],
    add_dashboard_middleware=True,
    use_test_urls=True,
).settings


def main():
    func_main(project_settings, f"{app_name}.tests")


if __name__ == "__main__":
    logging.basicConfig()
    main()



class AlreadyRegistered(Exception):
    pass


class InvalidUrlName(Exception):
    pass


class UrlNames:
    def __init__(self):
        self.registry = {}

    def __call__(self):
        return self.registry

    def __str__(self):
        return self.registry

    def register(self, name=None, url=None, namespace=None):
        name = name or url
        complete_url = f"{namespace}:{url}" if namespace else url
        if name in self.registry:
            raise AlreadyRegistered(f"Url already registered. Got {name}.")
        self.registry.update({name: complete_url})

    def register_from_dict(self, **urldata):
        for name, complete_url in urldata.items():
            try:
                namespace, url = complete_url.split(":")
            except ValueError:
                namespace, url = complete_url, None
            self.register(name=name, url=url, namespace=namespace)

    def get(self, name):
        if name not in self.registry:
            raise InvalidUrlName(
                "Invalid url name. Expected one of " f"{self.registry.keys()}. Got '{name}'."
            )
        return self.registry.get(name)


url_names = UrlNames()



from edc_protocol import Protocol
from edc_utils.text import convert_from_camel

from ..url_config import UrlConfig
from ..url_names import InvalidUrlName, url_names


class UrlRequestContextError(Exception):
    pass


class UrlRequestContextMixin:
    urlconfig_getattr = "dashboard_urls"
    urlconfig_identifier_label = "subject_identifier"
    urlconfig_identifier_pattern = Protocol().subject_identifier_pattern
    urlconfig_label = None
    url_name = None

    @classmethod
    def get_urlname(cls):
        return cls.url_name

    @classmethod
    def urls(cls, namespace=None, label=None, identifier_label=None, identifier_pattern=None):
        label = (
            label
            or cls.urlconfig_label
            or convert_from_camel(cls.__name__.replace("view", "")).lower()
        )
        urlconfig = UrlConfig(
            url_name=cls.get_urlname(),
            namespace=namespace,
            view_class=cls,
            label=label,
            identifier_label=identifier_label or cls.urlconfig_identifier_label,
            identifier_pattern=identifier_pattern or cls.urlconfig_identifier_pattern,
        )
        return getattr(urlconfig, cls.urlconfig_getattr)

    @staticmethod
    def add_url_to_context(new_key=None, existing_key=None, context=None):
        """Add url as new_key to the context using the value
        of the existing_key from request.context_data.
        """
        try:
            context.update({new_key: url_names.get(existing_key)})
        except InvalidUrlName as e:
            raise UrlRequestContextError(
                f"Url name not defined in url_names. "
                f"Expected one of {url_names.registry}. Got {e}. "
                f"Hint: check if dashboard middleware is loaded."
            )
        return context



from datetime import datetime
from zoneinfo import ZoneInfo

import arrow
from django.contrib.auth.models import Group, User
from django.contrib.sites.models import Site
from django.test import TestCase, override_settings
from django.test.client import RequestFactory
from django.views.generic.base import ContextMixin, View
from edc_auth.auth_objects import CLINIC
from edc_auth.auth_updater import AuthUpdater
from edc_auth.site_auths import site_auths
from edc_listboard.filters import ListboardFilter, ListboardViewFilters
from edc_listboard.view_mixins import ListboardFilterViewMixin, QueryStringViewMixin
from edc_listboard.views import ListboardView
from edc_model_wrapper import ModelWrapper
from edc_utils import get_utcnow

from edc_dashboard.url_names import url_names

from ..models import SubjectVisit


@override_settings(EDC_AUTH_SKIP_SITE_AUTHS=True, EDC_AUTH_SKIP_AUTH_UPDATER=False)
class TestViewMixins(TestCase):
    @classmethod
    def setUpTestData(cls):
        url_names.register("dashboard_url", "dashboard_url", "edc_dashboard")
        site_auths.clear()
        site_auths.add_group("edc_dashboard.view_my_listboard", name=CLINIC)
        site_auths.add_custom_permissions_tuples(
            model="edc_dashboard.edcpermissions",
            codename_tuples=(("edc_dashboard.view_my_listboard", "View my listboard"),),
        )
        AuthUpdater(verbose=False, warn_only=True)
        return super().setUpTestData()

    def setUp(self):
        self.user = User.objects.create(username="erik")
        group = Group.objects.get(name=CLINIC)
        self.user.groups.add(group)
        self.request = RequestFactory().get("/")
        self.request.user = self.user

    def test_querystring_mixin(self):
        class MyView(QueryStringViewMixin, ContextMixin, View):
            pass

        request = RequestFactory().get("/?f=f&e=e&o=o&q=q")
        request.user = self.user
        view = MyView(request=request)
        self.assertIn("f=f", view.querystring)
        self.assertIn("e=e", view.querystring)
        self.assertIn("o=o", view.querystring)
        self.assertIn("q=q", view.querystring)
        for attr in ["f", "e", "o", "q"]:
            with self.subTest(attr=attr):
                self.assertEqual(attr, view.get_context_data().get(attr), attr)

    def test_listboard_filter_view(self):
        class SubjectVisitModelWrapper(ModelWrapper):
            model = "edc_dashboard.subjectvisit"
            next_url_name = "dashboard_url"

        class MyListboardViewFilters(ListboardViewFilters):
            all = ListboardFilter(name="all", label="All", lookup={})

            scheduled = ListboardFilter(label="Scheduled", lookup={"reason": "scheduled"})

            not_scheduled = ListboardFilter(
                label="Not Scheduled",
                exclude_filter=True,
                lookup={"reason": "scheduled"},
            )

        class MyView(ListboardFilterViewMixin, ListboardView):
            listboard_model = "edc_dashboard.subjectvisit"
            listboard_url = "listboard_url"
            listboard_template = "listboard_template"
            listboard_filter_url = "listboard_url"
            listboard_view_permission_codename = "edc_dashboard.view_my_listboard"
            model_wrapper_cls = SubjectVisitModelWrapper
            listboard_view_filters = MyListboardViewFilters()

        start = datetime(2013, 5, 1, 12, 30, tzinfo=ZoneInfo("UTC"))
        end = datetime(2013, 5, 10, 17, 15, tzinfo=ZoneInfo("UTC"))
        for arr in arrow.Arrow.range("day", start, end):
            SubjectVisit.objects.create(
                subject_identifier="1234", report_datetime=arr.datetime, reason="missed"
            )
        subject_visit = SubjectVisit.objects.create(
            subject_identifier="1234", report_datetime=get_utcnow(), reason="scheduled"
        )
        request = RequestFactory().get("/?scheduled=scheduled")
        request.user = self.user
        request.site = Site.objects.get_current()
        request.template_data = {"listboard_template": "listboard.html"}
        template_response = MyView.as_view()(request=request)
        object_list = template_response.__dict__.get("context_data").get("object_list")
        self.assertEqual(
            [
                wrapper.object.reason
                for wrapper in object_list
                if wrapper.object.pk == subject_visit.pk
            ],
            [subject_visit.reason],
        )



from edc_auth.site_auths import site_auths
from edc_auth.utils import remove_default_model_permissions_from_edc_permissions

from .auth_objects import dashboard_tuples

site_auths.add_post_update_func(
    "edc_dashboard", remove_default_model_permissions_from_edc_permissions
)


site_auths.add_custom_permissions_tuples(
    model="edc_dashboard.edcpermissions", codename_tuples=dashboard_tuples
)



from importlib.metadata import version

__version__ = version("edc_dashboard")

from django.conf import settings
from django.core.exceptions import ImproperlyConfigured

from .url_config import UrlConfig
from .url_names import url_names
from .utils import insert_bootstrap_version, select_edc_template

name = "edc_dashboard.middleware.DashboardMiddleware"
if name not in settings.MIDDLEWARE:
    raise ImproperlyConfigured(f"Missing middleware. Expected {name}.")



from django.core.exceptions import ImproperlyConfigured
from django.views.generic.base import TemplateView

from ..url_names import url_names
from ..view_mixins import TemplateRequestContextMixin, UrlRequestContextMixin


class DashboardView(UrlRequestContextMixin, TemplateRequestContextMixin, TemplateView):
    dashboard_url_name = None
    dashboard_template = None  # may be None if `dashboard_template_name` is defined
    dashboard_template_name = None  # may be None if `dashboard_template` is defined

    urlconfig_getattr = "dashboard_urls"

    def __init__(self, **kwargs):
        if not self.dashboard_template and not self.dashboard_template_name:
            raise ImproperlyConfigured(
                f"Both 'dashboard_template' and 'dashboard_template_name' "
                f"cannot be None. See {repr(self)}."
            )
        super().__init__(**kwargs)

    @classmethod
    def get_urlname(cls):
        return cls.dashboard_url_name

    @property
    def dashboard_url(self):
        return url_names.get(self.dashboard_url_name)

    def get_template_names(self):
        if self.dashboard_template_name:
            return [self.dashboard_template_name]
        return [self.get_template_from_context(self.dashboard_template)]

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context = self.add_url_to_context(
            new_key="dashboard_url_name",
            existing_key=self.dashboard_url_name,
            context=context,
        )
        return context



from django.urls.conf import path
from django.views.generic.base import RedirectView
from edc_listboard.views import ListboardView
from edc_utils.paths_for_urlpatterns import paths_for_urlpatterns

from ..url_config import UrlConfig
from ..views import AdministrationView
from .admin import edc_dashboard_admin

app_name = "edc_dashboard"

subject_listboard_url_config = UrlConfig(
    url_name="listboard_url",
    namespace=app_name,
    view_class=ListboardView,
    label="subject_listboard",
    identifier_label="subject_identifier",
    identifier_pattern="/w+",
)


urlpatterns = subject_listboard_url_config.listboard_urls

for app_name in [
    "edc_dashboard",
    "edc_auth",
    "edc_adverse_event",
    "edc_randomization",
    "edc_consent",
    "edc_export",
    "edc_device",
    "edc_protocol",
    "edc_reference",
    "edc_visit_schedule",
]:
    for p in paths_for_urlpatterns(app_name):
        urlpatterns.append(p)

urlpatterns += [
    path("admin/", edc_dashboard_admin.urls),
    path("administration/", AdministrationView.as_view(), name="administration_url"),
    path("", RedirectView.as_view(url="admin/"), name="home_url"),
    path("", RedirectView.as_view(url="admin/"), name="logout"),
]



from dateutil.relativedelta import relativedelta
from django.core.paginator import Paginator
from django.test import TestCase
from edc_utils.date import get_utcnow

from edc_dashboard.templatetags.edc_dashboard_extras import (
    age_in_years,
    human,
    page_numbers,
    paginator_row,
)

from ..models import TestModel


class TestTags(TestCase):
    def test_(self):
        for i in range(0, 100):
            TestModel.objects.create(f1=f"object{i}")

        object_list = TestModel.objects.all()
        paginator = Paginator(object_list, 5)

        page = 1
        context = dict(
            paginator=paginator,
            page_obj=paginator.get_page(page),
            query_string=None,
            paginator_url="listboard_url",
        )
        context = paginator_row(context)
        self.assertIsNone(context.get("first_url"))
        self.assertIsNone(context.get("previous_url"))
        self.assertEqual(context.get("next_url"), "/subject_listboard/2/")
        self.assertEqual(context.get("last_url"), "/subject_listboard/20/")
        self.assertEqual(context.get("numbers")[0].number, 1)
        self.assertEqual(context.get("numbers")[-1:][0].number, 10)

        page = 2
        context = dict(
            paginator=paginator,
            page_obj=paginator.get_page(page),
            query_string=None,
            paginator_url="listboard_url",
        )
        context = paginator_row(context)
        self.assertEqual(context.get("first_url"), "/subject_listboard/1/")
        self.assertEqual(context.get("previous_url"), "/subject_listboard/1/")
        self.assertEqual(context.get("next_url"), "/subject_listboard/3/")
        self.assertEqual(context.get("last_url"), "/subject_listboard/20/")
        self.assertEqual(context.get("numbers")[0].number, 1)
        self.assertEqual(context.get("numbers")[-1:][0].number, 10)

        page = 3
        context = dict(
            paginator=paginator,
            page_obj=paginator.get_page(page),
            query_string=None,
            paginator_url="listboard_url",
        )
        context = paginator_row(context)
        self.assertEqual(context.get("first_url"), "/subject_listboard/1/")
        self.assertEqual(context.get("previous_url"), "/subject_listboard/2/")
        self.assertEqual(context.get("next_url"), "/subject_listboard/4/")
        self.assertEqual(context.get("last_url"), "/subject_listboard/20/")
        self.assertEqual(context.get("numbers")[0].number, 1)
        self.assertEqual(context.get("numbers")[-1:][0].number, 10)

        page = 20
        context = dict(
            paginator=paginator,
            page_obj=paginator.get_page(page),
            query_string=None,
            paginator_url="listboard_url",
        )
        context = paginator_row(context)
        self.assertEqual(context.get("first_url"), "/subject_listboard/1/")
        self.assertEqual(context.get("previous_url"), "/subject_listboard/19/")
        self.assertIsNone(context.get("next_url"))
        self.assertIsNone(context.get("last_url"))
        self.assertEqual(context.get("numbers")[0].number, 15)
        self.assertEqual(context.get("numbers")[-1:][0].number, 20)

    def test_page_numbers(self):
        for i in range(1, 25):
            self.assertEqual(len(page_numbers(i, 200)), 10)
            self.assertIn(i, page_numbers(i, 200))

    def test_age(self):
        context = {"reference_datetime": None}
        born = get_utcnow() - relativedelta(years=25)
        self.assertEqual(25, age_in_years(context, born))

        reference_datetime = get_utcnow() - relativedelta(years=25)
        context = {"reference_datetime": reference_datetime}
        born = reference_datetime - relativedelta(years=5)
        self.assertEqual(5, age_in_years(context, born))

        reference_datetime = get_utcnow() - relativedelta(years=25)
        context = {"reference_datetime": reference_datetime}
        born = get_utcnow()
        self.assertEqual(born, age_in_years(context, born))

    def test_human(self):
        self.assertEqual(human(11112222333344445555), "1111-2222-3333-4444-5555")



import os
from warnings import warn

from django.apps import apps as django_apps
from django.conf import settings
from django.template.exceptions import TemplateDoesNotExist
from django.template.loader import get_template, select_template


class EdcTemplateDoesNotExist(Exception):
    pass


def get_bootstrap_version() -> int:
    return getattr(settings, "EDC_BOOTSTRAP", 3)


def get_index_page() -> int:
    index_page = getattr(settings, "INDEX_PAGE", None)
    if not index_page:
        warn("Settings attribute not set. See settings.INDEX_PAGE")
    return getattr(settings, "INDEX_PAGE", None)


def get_index_page_label() -> int:
    return getattr(settings, "INDEX_PAGE_LABLE", settings.APP_NAME)


def insert_bootstrap_version(**template_data):
    """Returns template data after inserting bootstrap version
    in each path, if not already inserted.
    """
    bootstrap_version = get_bootstrap_version()
    if bootstrap_version:
        for key, original_path in template_data.items():
            try:
                get_template(original_path)
            except TemplateDoesNotExist:
                if "bootstrap" not in original_path:
                    path = get_template_path_with_bootstrap(original_path, bootstrap_version)
                    template_data.update({key: path})
    return template_data


def get_template_path_with_bootstrap(original_path, bootstrap_version=None):
    """Returns a new path with the bootstrap version inserted
    or raises EdcTemplateDoesNotExist.
    """
    app_name = None
    bootstrap_version = bootstrap_version or get_bootstrap_version()
    for app_config in django_apps.get_app_configs():
        if app_config.name in original_path:
            app_name = app_config.name
            break
    if not app_name:
        raise EdcTemplateDoesNotExist(
            f"Template file path refers to unknown app_name. "
            f"Is the app listed in INSTALLED_APPS? "
            f"Is the app_config.name correctly loaded and set? "
            f"Got {original_path}"
        )
    else:
        path_list = splitall(original_path)
        path_list.insert(1, f"bootstrap{bootstrap_version}")
        path = os.path.join(*path_list)
        try:
            get_template(path)
        except TemplateDoesNotExist as e:
            raise EdcTemplateDoesNotExist(
                f"Template file does not exist. " f"Tried {original_path} and {path}. Got {e}"
            )
    return path


def splitall(path):
    """Taken from
    https://www.oreilly.com/library/view/python-cookbook/0596001673/ch04s16.html
    """
    allparts = []
    while 1:
        parts = os.path.split(path)
        if parts[0] == path:  # sentinel for absolute paths
            allparts.insert(0, parts[0])
            break
        elif parts[1] == path:  # sentinel for relative paths
            allparts.insert(0, parts[1])
            break
        else:
            path = parts[0]
            allparts.insert(0, parts[1])
    return allparts


def select_edc_template(relative_path, default_app_label):
    """Returns a template object."""
    local_path = f"{settings.APP_NAME}/bootstrap{get_bootstrap_version()}/"
    default_path = f"{default_app_label}/bootstrap{get_bootstrap_version()}/"
    return select_template(
        [
            os.path.join(local_path, relative_path),
            os.path.join(default_path, relative_path),
        ]
    )


def get_dashboard_app_label():
    return getattr(settings, "EDC_DASHBOARD_APP_LABEL", None)



from django.contrib import admin
from django.contrib.admin import AdminSite as DjangoAdminSite

from .models import SubjectVisit


class AdminSite(DjangoAdminSite):
    site_title = "Example"
    site_header = "Example"
    index_title = "Example"
    site_url = "/"


edc_dashboard_admin = AdminSite(name="edc_dashboard_admin")


@admin.register(SubjectVisit, site=edc_dashboard_admin)
class SubjectVisitAdmin(admin.ModelAdmin):
    pass



from django.contrib import messages
from django.utils.html import format_html


class MessageViewMixin:
    def message_user(self, message, level=None, extra_tags=None, fail_silently=None):
        """
        Send a message to the user. The default implementation
        posts a message using the django.contrib.messages backend.

        Exposes almost the same API as messages.add_message(), but accepts the
        positional arguments in a different order to maintain backwards
        compatibility. For convenience, it accepts the `level` argument as
        a string rather than the usual level number.
        """
        level = level or messages.ERROR
        extra_tags = extra_tags or ""
        if not isinstance(level, int):
            # attempt to get the level if passed a string
            try:
                level = getattr(messages.constants, level.upper())
            except AttributeError:
                levels = ", ".join(
                    [level for level in messages.constants.DEFAULT_TAGS.values()]
                )
                raise ValueError(
                    f"Bad message level string: `{level}`. Possible values are: {levels}"
                )

        messages.add_message(
            self.request,
            level,
            format_html(message),
            extra_tags=extra_tags,
            fail_silently=fail_silently,
        )



from django.db import models
from edc_model.models import BaseUuidModel, HistoricalRecords
from edc_search.model_mixins import SearchSlugManager, SearchSlugModelMixin
from edc_sites.models import SiteModelMixin


class SubjectVisit(SiteModelMixin, SearchSlugModelMixin, BaseUuidModel):
    subject_identifier = models.CharField(max_length=25, null=True)

    report_datetime = models.DateTimeField()

    reason = models.CharField(max_length=25, null=True)

    objects = SearchSlugManager()

    def get_search_slug_fields(self):
        fields = ["subject_identifier", "reason"]
        return fields


class TestModel(models.Model):
    f1 = models.CharField(max_length=25, null=True)

    class Meta:
        ordering = ("f1",)


class SubjectConsent(models.Model):
    name = models.CharField(max_length=50, null=True)

    history = HistoricalRecords()


class SubjectReconsent(models.Model):
    name = models.CharField(max_length=50, null=True)

    history = HistoricalRecords()


class SubjectRequisition(models.Model):
    name = models.CharField(max_length=50, null=True)

    history = HistoricalRecords()



from django.urls.conf import re_path
from edc_constants.constants import UUID_PATTERN

from .url_names import url_names


class UrlConfig:
    def __init__(
        self,
        url_name=None,
        namespace=None,
        view_class=None,
        label=None,
        identifier_label=None,
        identifier_pattern=None,
    ):
        self.identifier_label = identifier_label
        self.identifier_pattern = identifier_pattern
        self.label = label
        self.url_name = url_name
        self.view_class = view_class

        # register {urlname, namespace:urlname} with url_names
        url_names.register(url=self.url_name, namespace=namespace)

    @property
    def dashboard_urls(self):
        """Returns url patterns."""
        urlpatterns = [
            re_path(
                f"{self.label}/"
                f"(?P<{self.identifier_label}>{self.identifier_pattern})/"
                r"(?P<visit_schedule_name>\w+)/"
                r"(?P<schedule_name>\w+)/"
                r"(?P<visit_code>\w+)/"
                r"(?P<unscheduled>\w+)/",
                self.view_class.as_view(),
                name=self.url_name,
            ),
            re_path(
                f"{self.label}/"
                f"(?P<{self.identifier_label}>{self.identifier_pattern})/"
                r"(?P<visit_schedule_name>\w+)/"
                r"(?P<schedule_name>\w+)/"
                r"(?P<visit_code>\w+)/",
                self.view_class.as_view(),
                name=self.url_name,
            ),
            re_path(
                f"{self.label}/"
                f"(?P<{self.identifier_label}>{self.identifier_pattern})/"
                f"(?P<appointment>{UUID_PATTERN.pattern})/"
                r"(?P<scanning>\d)/"
                r"(?P<error>\d)/",
                self.view_class.as_view(),
                name=self.url_name,
            ),
            re_path(
                f"{self.label}/"
                f"(?P<{self.identifier_label}>{self.identifier_pattern})/"
                f"(?P<appointment>{UUID_PATTERN.pattern})/"
                r"(?P<reason>\w+)/",
                self.view_class.as_view(),
                name=self.url_name,
            ),
            re_path(
                f"{self.label}/"
                f"(?P<{self.identifier_label}>{self.identifier_pattern})/"
                f"(?P<appointment>{UUID_PATTERN.pattern})/",
                self.view_class.as_view(),
                name=self.url_name,
            ),
            re_path(
                f"{self.label}/"
                f"(?P<{self.identifier_label}>{self.identifier_pattern})/"
                r"(?P<schedule_name>\w+)/",
                self.view_class.as_view(),
                name=self.url_name,
            ),
            re_path(
                f"{self.label}/" f"(?P<{self.identifier_label}>{self.identifier_pattern})/",
                self.view_class.as_view(),
                name=self.url_name,
            ),
        ]
        return urlpatterns

    @property
    def listboard_urls(self):
        """Returns url patterns.

        configs = [(listboard_url, listboard_view_class, label), (), ...]
        """
        urlpatterns = [
            re_path(
                f"{self.label}/"
                f"(?P<{self.identifier_label}>{self.identifier_pattern})/"
                r"(?P<page>\d+)/",
                self.view_class.as_view(),
                name=self.url_name,
            ),
            re_path(
                f"{self.label}/" f"(?P<{self.identifier_label}>{self.identifier_pattern})/",
                self.view_class.as_view(),
                name=self.url_name,
            ),
            re_path(
                f"{self.label}" + r"/(?P<page>\d+)/",
                self.view_class.as_view(),
                name=self.url_name,
            ),
            re_path(f"{self.label}/", self.view_class.as_view(), name=self.url_name),
        ]
        return urlpatterns

    @property
    def review_listboard_urls(self):
        url_patterns = [
            re_path(
                f"{self.label}/"
                f"(?P<{self.identifier_label}>{self.identifier_pattern})/"
                f"(?P<appointment>{UUID_PATTERN.pattern})/",
                self.view_class.as_view(),
                name=self.url_name,
            )
        ]
        url_patterns.extend(self.listboard_urls)
        return url_patterns
